@import url('default.css');

.toctree-l1.current > a {
    color: #ffffff;
    font-weight: bold;
}

.toctree-l2.current > a {
    color: #efefff;
}

.toctree-l1.current > ul.current {
    margin-bottom: 0.6em;
}

li.toctree-l1 {
    margin-bottom: 0.2em;
}

ul.simple>li {
    margin-bottom: 0.35em;
}
